static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="org.gnome.SessionManager.Presence">
        <method name="SetStatus">
            <arg type="u" name="status" direction="in">
                <summary>The status value</summary>
            </arg>
            <description>Set the status value of the session.</description>
        </method>

        <method name="SetStatusText">
            <arg type="s" name="status_text" direction="in">
                <summary>The descriptive status for the session.</summary>
            </arg>
            <description>Set the descriptive status text for the session.</description>
        </method>

        <property name="status" type="u" access="readwrite">
            <description>The status of the session.</description>
        </property>

        <property name="status_text" type="s" access="readwrite">
            <description>The descriptive status for the session.</description>
        </property>

        <signal name="StatusChanged">
            <arg name="status" type="u">
                <summary>The new status value</summary>
            </arg>
            <description>Indicates that the session status value has changed.</description>
        </signal>

        <signal name="StatusTextChanged">
            <arg name="status_text" type="s">
                <summary>The new status text</summary>
            </arg>
            <description>Indicates that the descriptive session status text has changed.</description>
        </signal>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-session-manager-2.1.2/generated/presence_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

gnome::SessionManager::PresenceStub::PresenceStub():
    m_interfaceName("org.gnome.SessionManager.Presence")
{
    StatusChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &PresenceStub::StatusChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    StatusChanged_selectiveSignal.connect(sigc::mem_fun(this, &PresenceStub::StatusChanged_emitter));
    StatusTextChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &PresenceStub::StatusTextChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    StatusTextChanged_selectiveSignal.connect(sigc::mem_fun(this, &PresenceStub::StatusTextChanged_emitter));
}

gnome::SessionManager::PresenceStub::~PresenceStub()
{
    unregister_object();
}

guint gnome::SessionManager::PresenceStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &PresenceStub::on_method_call),
            sigc::mem_fun(this, &PresenceStub::on_interface_get_property),
            sigc::mem_fun(this, &PresenceStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("org.gnome.SessionManager.Presence"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void gnome::SessionManager::PresenceStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void gnome::SessionManager::PresenceStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("SetStatus") == 0) {
        Glib::Variant<guint32> base_status;
        parameters.get_child(base_status, 0);
        guint32 p_status = specialGetter(base_status);

        MethodInvocation methodInvocation(invocation);
        SetStatus(
            (p_status),
            methodInvocation);
    }

    if (method_name.compare("SetStatusText") == 0) {
        Glib::Variant<Glib::ustring> base_status_text;
        parameters.get_child(base_status_text, 0);
        Glib::ustring p_status_text = specialGetter(base_status_text);

        MethodInvocation methodInvocation(invocation);
        SetStatusText(
            (p_status_text),
            methodInvocation);
    }

}

void gnome::SessionManager::PresenceStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

    if (property_name.compare("status") == 0) {

        property = Glib::Variant<guint32>::create((status_get()));
    }

    if (property_name.compare("status_text") == 0) {

        property = Glib::Variant<Glib::ustring>::create((status_text_get()));
    }

}

bool gnome::SessionManager::PresenceStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    if (property_name.compare("status") == 0) {
        try {
            Glib::Variant<guint32> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<guint32>>(value);
            guint32 val =
                (specialGetter(castValue));
            status_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting status");
        }
    }

    if (property_name.compare("status_text") == 0) {
        try {
            Glib::Variant<Glib::ustring> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<Glib::ustring>>(value);
            Glib::ustring val =
                (specialGetter(castValue));
            status_text_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting status_text");
        }
    }

    return true;
}

void gnome::SessionManager::PresenceStub::StatusChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guint32 status)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guint32>::create((status)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.Presence",
                    "StatusChanged",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManager::PresenceStub::StatusTextChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,const Glib::ustring & status_text)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<Glib::ustring>::create((status_text)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.Presence",
                    "StatusTextChanged",
                    bus_name,
                    params);
        }
    }
}


bool gnome::SessionManager::PresenceStub::status_set(guint32 value)
{
    if (status_setHandler(value)) {
        Glib::Variant<guint32> value_get =
            Glib::Variant<guint32>::create((status_get()));

        emitSignal("status", value_get);
        return true;
    }

    return false;
}

bool gnome::SessionManager::PresenceStub::status_text_set(const Glib::ustring & value)
{
    if (status_text_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((status_text_get()));

        emitSignal("status_text", value_get);
        return true;
    }

    return false;
}

bool gnome::SessionManager::PresenceStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
